#############################################################################
# srptinfo.tcl Copyright 2004, 2005 r0lZ
# show/edit VMG_TT_SRPT table.
#
# This file is part of PgcEdit.
#
# PgcEdit is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# PgcEdit is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#############################################################################

namespace eval ::srptinfo {

namespace export srptinfo

proc srptinfo {} {
	##### get the table of titles from the VMG_TT_SRPT
	if {(! [info exists ::dvddir]) || ($::raw_sectors(0,VMG_TT_SRPT) == "")} {
		bell
		return
	}

	set w ".srptinfo"
	if {![winfo exists $w]} {
		toplevel $w -class Dialog
		wm withdraw $w
		wm iconname $w Dialog
		wm protocol $w WM_DELETE_WINDOW {set ::srptinfo::done 0}
		bind $w <Escape> "set ::srptinfo::done 0"
		wm transient $w .
		if {$::tcl_platform(platform) == "windows"} {catch {wm attributes $w -toolwindow 1}}
		wm title $w "Title Play Map table"

		frame  $w.mf -borderwidth 2 -relief groove
		pack   $w.mf -padx 4 -pady 4 -expand 1 -fill both
		canvas $w.mf.c -yscrollcommand "$w.mf.vs set" -height 10
		pack   $w.mf.c -side left -expand 1 -fill both
		scrollbar $w.mf.vs -command "$w.mf.c yview" -orient vert
		pack   $w.mf.vs -side left -fill y
		frame  $w.mf.c.f1	;# destroyed later

		# buttons
		frame  $w.fb
		pack   $w.fb -padx 2 -pady 4 -fill x
		button $w.fb.cl -text "Clear Prohibited User Operations" \
				-command {::srptinfo::clear_puos}
		pack   $w.fb.cl -side left -padx 2
		frame  $w.fb.pad
		pack   $w.fb.pad -expand 1 -fill x -side left
		button $w.fb.ok -text "OK" -width 10 -command {set ::srptinfo::done 1}
		pack   $w.fb.ok -side left -padx 3
		button $w.fb.cancel -text "Cancel" -width 10 -command {set ::srptinfo::done 0}
		pack   $w.fb.cancel -side left -padx 2

		set win_created true
	} else {
		set win_created false
	}

	destroy $w.mf.c.f1
	frame $w.mf.c.f1
	pack  $w.mf.c.f1 -anchor nw -fill both -expand 1
	$w.mf.c create window 0 0 -window $w.mf.c.f1 -anchor nw

	label $w.mf.c.f1.w_0_0 -width 8 -text "Title"
	label $w.mf.c.f1.w_0_1 -width 8 -text "VTS"
	label $w.mf.c.f1.w_0_2 -width 8 -text "TTN"
	label $w.mf.c.f1.w_0_3 -width 8 -text "Chapters\n(PTTs)"
	label $w.mf.c.f1.w_0_4 -width 8 -text "Angles"
	label $w.mf.c.f1.w_0_5 -width 8 -text "Parental\nmanag.\nmask"
	label $w.mf.c.f1.w_0_6 -width 8 -text "Sequential\nPGC"
	label $w.mf.c.f1.w_0_7 -width 8 -text "PUO:\nTime play\nor search"
	label $w.mf.c.f1.w_0_8 -width 8 -text "PUO:\nPTT play\nor search"

	set data $::raw_sectors(0,VMG_TT_SRPT)
	set ::srptinfo::numtitles [get_int data 0 2]
	for {set t 1} {$t <= $::srptinfo::numtitles} {incr t} {
		set t_idx [expr 8+(($t-1)*12)]

		# Title type flags
		set type [get_int data $t_idx 1]
		set ::srptinfo::title($t,uop0) [expr  $type & 1]
		set ::srptinfo::title($t,uop1) [expr ($type & 2) / 2]
		if [expr ($type & 0X80) / 0X80] {
			set sequential "no"
		} else {
			set sequential "yes"
		}

		label $w.mf.c.f1.w_${t}_0 -text $t
		label $w.mf.c.f1.w_${t}_1 -text [get_int data [expr $t_idx+6] 1]
		label $w.mf.c.f1.w_${t}_2 -text [get_int data [expr $t_idx+7] 1]
		label $w.mf.c.f1.w_${t}_3 -text [get_int data [expr $t_idx+2] 2]
		label $w.mf.c.f1.w_${t}_4 -text [get_int data [expr $t_idx+1] 1]
		label $w.mf.c.f1.w_${t}_5 -text [get_int data [expr $t_idx+4] 2]
		label $w.mf.c.f1.w_${t}_6 -text $sequential
		checkbutton $w.mf.c.f1.w_${t}_7 -variable ::srptinfo::title($t,uop0)
		checkbutton $w.mf.c.f1.w_${t}_8 -variable ::srptinfo::title($t,uop1)
	}
	for {set t 0} {$t <= $::srptinfo::numtitles} {incr t} {
		for {set f 0} {$f <= 8} {incr f} {
			grid $w.mf.c.f1.w_${t}_$f -row $t -column $f
		}
	}

	# inform scrollbar of the height of the canvas's internal window
	update
	$w.mf.c configure -scrollregion "0 0 0 [winfo height $w.mf.c.f1]" \
			-width [winfo width $w.mf.c.f1]

	# window geometry
	if $win_created {
		set maxheight [winfo height $w.mf.c.f1]
		if {$maxheight > 304} {set maxheight 304}
		$w.mf.c configure -height $maxheight
		update
		set ww [winfo reqwidth $w]
		set wh [winfo reqheight $w]
		set wx [expr [winfo rootx .]+(([winfo width  .]-$ww)/2)]
		set wy [expr [winfo rooty .]+(([winfo height .]-$wh)/2)-25]
		if {$wx < 10} {set wx 10}
		if {$wy < 10} {set wy 10}
		$w.mf.c configure -height 1
		wm geometry $w ${ww}x$wh+$wx+$wy
		wm resizable $w 0 1
	}

	wm deiconify $w
	focus $w
	grab set $w

	set ::srptinfo::done -1
	tkwait variable ::srptinfo::done
	grab release $w
	destroy $w

	if $::srptinfo::done {
		::main::dvd_modified true false
		for {set t 1} {$t <= $::srptinfo::numtitles} {incr t} {
			set t_idx [expr 8+(($t-1)*12)]

			# Title type flags
			set type [get_int data $t_idx 1]
			if $::srptinfo::title($t,uop0) {
				set type [expr $type | 1]
			} else {
				set type [expr $type & 0XFE]
			}
			if $::srptinfo::title($t,uop1) {
				set type [expr $type | 2]
			} else {
				set type [expr $type & 0XFD]
			}
			set_int ::raw_sectors(0,VMG_TT_SRPT) $t_idx $type 1
		}
	}
	return
}

proc clear_puos {} {
	for {set t 0} {$t <= $::srptinfo::numtitles} {incr t} {
		set ::srptinfo::title($t,uop0) 0
		set ::srptinfo::title($t,uop1) 0
	}
}

proc srptinfo_dump {} {
	##### get the table of titles from the VMG_TT_SRPT
	if {(! [info exists ::dvddir]) || ($::raw_sectors(0,VMG_TT_SRPT) == "")} {
		bell
		return
	}

	set txt "Title Play Map table\n"
	set txt "${txt}DVD-TEXT General Name: \"[::utils::get_General_Name]\"\n"
	set txt "${txt}Provider ID: \"[::utils::get_provider_ID]\"\n"
	set txt "${txt}Number of VTS: $::pgcs(numvts)\n\n"

	set txt "${txt}Title  VTS   TTN  Chapters Angles Parental   Sequential    PUO:        PUO:\n"
	set txt "${txt}                   (PTTs)         Managment     PGC      Time Play   PTT Play\n"
	set txt "${txt}                                  Mask (hex)             or search   or search\n\n"

	set data $::raw_sectors(0,VMG_TT_SRPT)
	set numtitles [get_int data 0 2]
	for {set t 1} {$t <= $numtitles} {incr t} {
		set t_idx [expr 8+(($t-1)*12)]

		# Title type flags
		set type [get_int data $t_idx 1]
		set uop0 "allowed"
		set uop1 "allowed"
		if {[expr  $type & 1] > 0} {set uop0 "prohibited"}
		if {[expr  $type & 2] > 0} {set uop1 "prohibited"}
		set title($t,uop1) [expr ($type & 2) / 2]
		if [expr ($type & 0X80) / 0X80] {
			set sequential "no"
		} else {
			set sequential "yes"
		}
		set txt $txt[format "%3d    %2d    %2d     %3d    %3d     0x%04X       %-3s      %-10s  %-10s" \
				$t [get_int data [expr $t_idx+6] 1] [get_int data [expr $t_idx+7] 1] \
				[get_int data [expr $t_idx+2] 2] [get_int data [expr $t_idx+1] 1] \
				[get_int data [expr $t_idx+4] 2] $sequential $uop0 $uop1 ]\n
	}
	::main::showtext .srptdump "Title Play Map Table" $txt
	return
}

} ;# end namespace srptinfo
