#############################################################################
# toolscfg.tcl Copyright 2004, 2005 r0lZ
# Tool Menu configuration GUI.
#
# This file is part of PgcEdit.
#
# PgcEdit is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# PgcEdit is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#############################################################################


namespace eval ::toolscfg {
namespace export toolscfg configure_tools_menu

#################################################
## Main window
## argument: window name.

proc toolscfg {w} {
	set ::toolscfg::toolslist  ""
	set ::toolscfg::toolsnames ""
	foreach t $::config(tools) {
		lappend ::toolscfg::toolslist  $t
		lappend ::toolscfg::toolsnames [lindex $t 0]
	}

	if [winfo exists $w] {destroy $w}
	toplevel $w -class Dialog
	wm withdraw $w
	wm title $w "Tools menu configuration"
	wm transient $w .
	wm protocol $w WM_DELETE_WINDOW {set ::toolscfg::done 0}
	bind $w <Escape> "set ::toolscfg::done 0"

	frame $w.top
	pack  $w.top -padx 8 -pady 4 -fill both -expand 1

	listbox $w.top.lb -listvariable ::toolscfg::toolsnames \
			-width 8 -height 8 -yscrollcommand "$w.top.yscr set" \
			-selectmode browse -exportselection 0 -activestyle none
	pack  $w.top.lb -side left -pady 4 -fill both -expand 1
	scrollbar $w.top.yscr -command "$w.top.lb yview"
	pack  $w.top.yscr -side left -pady 4 -fill y
	bind $w.top.lb <1> "focus $w.top.lb"
	bind $w.top.lb <Double-1> "::toolscfg::edit $w.top.lb"
	bind $w.top.lb <Delete>   "::toolscfg::remove $w.top.lb"
	bind $w.top.lb <Insert>   "::toolscfg::add $w.top.lb"
	bind $w.top.lb <Return>   "::toolscfg::edit $w.top.lb"
	bind $w.top.lb <space>    "::toolscfg::edit $w.top.lb"
	bind $w.top.lb <Control-bracketleft>  "::toolscfg::moveup $w.top.lb"
	bind $w.top.lb <Control-bracketright> "::toolscfg::movedown $w.top.lb"
	bind $w.top.lb <Home> "$w.top.lb see 0"
	bind $w.top.lb <End>  "$w.top.lb see end"
	bind $w.top.lb <MouseWheel> "
		if {%D > 0} {
			$w.top.lb yview scroll  1 units
		} else {
			$w.top.lb yview scroll -1 units
		}
	"

	frame $w.top.pad -width 8
	pack  $w.top.pad -side left

	frame $w.top.fb -borderwidth 2
	pack  $w.top.fb -side left -pady 2 -fill y

	button $w.top.fb.edit   -width 10 -text "Edit" \
			-command "::toolscfg::edit $w.top.lb"
	button $w.top.fb.test   -width 10 -text "Test" \
			-command "::toolscfg::test $w.top.lb"
	button $w.top.fb.add    -width 10 -text "Add new" \
			-command "::toolscfg::add $w.top.lb"
	button $w.top.fb.sep    -width 10 -text "Separator" \
			-command "::toolscfg::separator $w.top.lb"
	button $w.top.fb.dup    -width 10 -text "Duplicate" \
			-command "::toolscfg::duplicate $w.top.lb"
	button $w.top.fb.del    -width 10 -text "Remove" \
			-command "::toolscfg::remove $w.top.lb"
	button $w.top.fb.moveup -width 10 -text "Move up" \
			-command "::toolscfg::moveup $w.top.lb"
	button $w.top.fb.movedn -width 10 -text "Move down" \
			-command "::toolscfg::movedown $w.top.lb"
	pack   $w.top.fb.edit   -side top -fill x -pady 2
	pack   $w.top.fb.test   -side top -fill x -pady 2
	pack   $w.top.fb.add    -side top -fill x -pady 2
	pack   $w.top.fb.sep    -side top -fill x -pady 2
	pack   $w.top.fb.dup    -side top -fill x -pady 2
	pack   $w.top.fb.del    -side top -fill x -pady 2
	pack   $w.top.fb.moveup -side top -fill x -pady 2
	pack   $w.top.fb.movedn -side top -fill x -pady 2

	frame $w.bot
	pack  $w.bot -pady 4 -fill x

	frame $w.bot.pad
	pack  $w.bot.pad -side left -fill x -expand 1

	button $w.bot.save -width 10 -command {set ::toolscfg::done 2} -text Save
	button $w.bot.use  -width 10 -command {set ::toolscfg::done 1} -text Use
	button $w.bot.can  -width 10 -command {set ::toolscfg::done 0} -text Cancel
	pack  $w.bot.save -side left -padx 10
	pack  $w.bot.use  -side left
	pack  $w.bot.can  -side left -padx 10

	update
	if {$::tcl_platform(platform) == "windows"} {
		catch {wm attributes $w -toolwindow 1}
	}
	wm minsize $w [winfo reqwidth $w] [winfo reqheight $w]
	$w.top.lb configure -width 30
	update
	set wx [expr [winfo rootx .]+(([winfo width  .]-[winfo reqwidth  $w])/2)]
	set wy [expr [winfo rooty .]+(([winfo height .]-[winfo reqheight $w])/2)-25]
	if {$wx < 10} {set wx 10}
	if {$wy < 10} {set wy 10}
	wm geometry $w [winfo reqwidth $w]x[winfo reqheight $w]+$wx+$wy
	$w.top.lb configure -width 8
	update

	wm deiconify $w
	grab set $w

	$w.top.lb activate 0
	$w.top.lb selection set 0
	focus $w.top.lb


	set ::toolscfg::done -1
	tkwait variable ::toolscfg::done
	grab release $w
	if {$::toolscfg::done > 0} {
		set ::config(tools) $::toolscfg::toolslist
		configure_tools_menu
	}
	if {$::toolscfg::done == 2} {
		save_user_tools $w
	}
	destroy $w
	return $::toolscfg::done
}
proc edit {w} {
	set idx [$w curselection]
	if {$idx != ""} {
		set cur [lindex $::toolscfg::toolslist $idx]
		if {[lindex $cur 0] != "---"} {
			set new [edititem $w $cur]
			if {$new != ""} {
				lset ::toolscfg::toolslist  $idx $new
				lset ::toolscfg::toolsnames $idx [lindex $new 0]
			}
		} else {
			bell
		}
	}
}
proc add {w} {
	set new [edititem $w {{} {} {} & {} false false false false false}]
	if {$new != ""} {
		lappend ::toolscfg::toolslist  $new
		lappend ::toolscfg::toolsnames [lindex $new 0]
		activateitem $w [expr [llength $::toolscfg::toolsnames]-1]
	}
}
proc separator {w} {
	lappend ::toolscfg::toolslist  {---}
	lappend ::toolscfg::toolsnames {---}
	activateitem $w [expr [llength $::toolscfg::toolsnames]-1]
}
proc duplicate {w} {
	set idx [$w curselection]
	if {$idx != ""} {
		set cur [lindex $::toolscfg::toolslist $idx]
		lappend ::toolscfg::toolslist  $cur
		lappend ::toolscfg::toolsnames [lindex $cur 0]
		activateitem $w [expr [llength $::toolscfg::toolsnames]-1]
	}
}
proc remove {w} {
	set idx [$w curselection]
	if {$idx != ""} {
		set ::toolscfg::toolslist  [lreplace $::toolscfg::toolslist  $idx $idx]
		set ::toolscfg::toolsnames [lreplace $::toolscfg::toolsnames $idx $idx]
		if {$idx == [llength $::toolscfg::toolsnames]} {
			activateitem $w [expr $idx -1]
		}
	}
}
proc moveup {w} {
	set idx [$w curselection]
	if {$idx != "" && $idx > 0} {
		set cur [lindex $::toolscfg::toolslist  $idx]
		set ::toolscfg::toolslist  [linsert [lreplace $::toolscfg::toolslist  $idx $idx] \
				[expr $idx-1] $cur]
		set cur [lindex $::toolscfg::toolsnames $idx]
		set ::toolscfg::toolsnames [linsert [lreplace $::toolscfg::toolsnames $idx $idx] \
				[expr $idx-1] $cur]
		activateitem $w [expr $idx-1]
	}
}
proc movedown {w} {
	set idx [$w curselection]
	if {$idx != "" && $idx < [expr [llength $::toolscfg::toolsnames]-1]} {
		set cur [lindex $::toolscfg::toolslist  $idx]
		set ::toolscfg::toolslist  [linsert [lreplace $::toolscfg::toolslist  $idx $idx] \
				[expr $idx+1] $cur]
		set cur [lindex $::toolscfg::toolsnames $idx]
		set ::toolscfg::toolsnames [linsert [lreplace $::toolscfg::toolsnames $idx $idx] \
				[expr $idx+1] $cur]
		activateitem $w [expr $idx+1]
	}
}
proc test {w} {
	set idx [$w curselection]
	if {$idx != ""} {
		if {[lindex [lindex $::toolscfg::toolslist $idx] 0] != "---"} {
			exec_tool [lindex $::toolscfg::toolslist $idx]
		} else {
			bell
		}
	}
}
proc activateitem {w idx} {
	$w activate $idx
	$w selection clear 0 end
	$w selection set $idx
	$w see $idx
}

#########################################################
## Edit item window

proc edititem {parent item} {
	set w ${parent}_edit
	if [winfo exists $w] {destroy $w}

	if {$item != ""} {
		set ::toolscfg::edit_e1 [lindex $item 0]
		set ::toolscfg::edit_e2 [lindex $item 1]
		set ::toolscfg::edit_e3 [lindex $item 2]
		set ::toolscfg::background [lindex $item 3]
		set ::toolscfg::edit_e4 [lindex $item 4]
		set ::toolscfg::confirm [lindex $item 5]
		set ::toolscfg::save    [lindex $item 6]
		set ::toolscfg::reopen  [lindex $item 7]
		set ::toolscfg::output  [lindex $item 8]
		set ::toolscfg::iconify [lindex $item 9]
	} else {
		set ::toolscfg::edit_e1 ""
		set ::toolscfg::edit_e2 ""
		set ::toolscfg::edit_e3 ""
		set ::toolscfg::background "&"
		set ::toolscfg::edit_e4 ""
		set ::toolscfg::confirm false
		set ::toolscfg::save    false
		set ::toolscfg::reopen  false
		set ::toolscfg::output  false
		set ::toolscfg::iconify false
	}

	toplevel $w -class Dialog
	wm withdraw $w
	wm title $w "Tool configuration"
	wm transient $w $parent
	wm protocol $w WM_DELETE_WINDOW {set ::toolscfg::edit_done 0}
	bind $w <Escape> "set ::toolscfg::edit_done 0"

	frame $w.top
	pack  $w.top -padx 8 -pady 4 -fill both -expand 1

	label $w.top.l1 -text "Label:"
	label $w.top.l2 -text "Application:"
	label $w.top.l3 -text "Arguments:"
	label $w.top.l4 -text "Working folder:"
	entry $w.top.e1 -textvariable ::toolscfg::edit_e1 -width 40
	entry $w.top.e2 -textvariable ::toolscfg::edit_e2 -width 40
	label $w.top.pad -text ""
	button $w.top.b2 -text "..." -command "::toolscfg::get_tool_filename $w" -pady 0 -width 2
	entry $w.top.e3 -textvariable ::toolscfg::edit_e3 -width 40 -font $::config(fixed_width_font)
	button $w.top.b3 -text "?" -command ::toolscfg::args_info -pady 0 -width 2
	entry $w.top.e4 -textvariable ::toolscfg::edit_e4 -width 40
	button $w.top.b4 -text "..." -command "::toolscfg::get_tool_workdir $w" -pady 0 -width 2
	grid  $w.top.l1 -column 0 -row 0 -sticky w
	grid  $w.top.l2 -column 0 -row 1 -sticky w
	grid  $w.top.l3 -column 0 -row 2 -sticky w
	grid  $w.top.l4 -column 0 -row 3 -sticky w
	grid  $w.top.e1 -column 1 -row 0 -sticky ew ;#-columnspan 3
	grid  $w.top.e2 -column 1 -row 1 -sticky ew
	grid  $w.top.e3 -column 1 -row 2 -sticky ew
	grid  $w.top.e4 -column 1 -row 3 -sticky ew
	grid  $w.top.pad -column 2 -row 1 -sticky ew
	grid  $w.top.b2 -column 3 -row 1 -sticky ew
	grid  $w.top.b3 -column 3 -row 2 -sticky ew
	grid  $w.top.b4 -column 3 -row 3 -sticky ew
	grid columnconf $w.top 1 -weight 1
	grid rowconf $w.top 1 -pad 8
	grid rowconf $w.top 3 -pad 8

	checkbutton $w.ck0 -text "Ask confirmation before launching tool ?" -variable ::toolscfg::confirm \
			-anchor w -onvalue true -offvalue false
	pack  $w.ck0 -fill x -padx 6 -anchor w
	checkbutton $w.ck1 -text "Save DVD before launching application ?" -variable ::toolscfg::save \
			-anchor w -onvalue true -offvalue false
	pack  $w.ck1 -fill x -padx 6 -anchor w
	checkbutton $w.ck2 -text "Run application in background ?" -variable ::toolscfg::background \
			-anchor w -onvalue "&" -offvalue "" -command "::toolscfg::update_checkboxes $w"
	pack  $w.ck2 -fill x -padx 6 -anchor w
	checkbutton $w.ck5 -text "Iconify PgcEdit when application is running ?" -variable ::toolscfg::iconify \
			-anchor w -onvalue true -offvalue false
	pack  $w.ck5 -fill x -padx 26 -pady 0 -anchor w
	checkbutton $w.ck4 -text "Display application output ?" -variable ::toolscfg::output \
			-anchor w -onvalue true -offvalue false
	pack  $w.ck4 -fill x -padx 26 -pady 0 -anchor w
	checkbutton $w.ck3 -text "Reopen DVD in PgcEdit after application is closed ?" -variable ::toolscfg::reopen \
			-anchor w -onvalue true -offvalue false
	pack  $w.ck3 -fill x -padx 26 -pady 0 -anchor w
	update_checkboxes $w

	frame $w.bot
	pack  $w.bot -pady 4 -fill x

	frame $w.bot.pad
	pack  $w.bot.pad -side left -fill x -expand 1

	button $w.bot.ok  -width 10 -command {set ::toolscfg::edit_done 1} -text OK
	button $w.bot.can -width 10 -command {set ::toolscfg::edit_done 0} -text Cancel
	pack  $w.bot.ok  -side left
	pack  $w.bot.can -side left -padx 8

	update
	if {$::tcl_platform(platform) == "windows"} {
		catch {wm attributes $w -toolwindow 1}
	}
	wm minsize $w [winfo reqwidth $w] [winfo reqheight $w]
	wm maxsize $w 800 [winfo reqheight $w]
	set wx [expr [winfo rootx $parent]+10]
	set wy [expr [winfo rooty $parent]+10]
	wm geometry $w +$wx+$wy

	wm deiconify $w
	focus $w.top.e1
	set oldgrab [grab current .]
	grab set $w

	set done 0
	while {! $done} {
		set ::toolscfg::edit_done -1
		tkwait variable ::toolscfg::edit_done
		if {$::toolscfg::edit_done} {
			if {$::toolscfg::edit_e1 != "" && $::toolscfg::edit_e2 != ""} {
				break
			} else {
				bell
			}
		} else {
			break
		}
	}
	grab release $w
	grab set $oldgrab
	destroy $w
	if $::toolscfg::edit_done {
		return [list $::toolscfg::edit_e1 $::toolscfg::edit_e2 $::toolscfg::edit_e3 \
				$::toolscfg::background $::toolscfg::edit_e4 $::toolscfg::confirm $::toolscfg::save \
				$::toolscfg::reopen $::toolscfg::output $::toolscfg::iconify]
	} else {
		return ""
	}
}
proc get_tool_filename {w} {
	if {$::tcl_platform(platform) == "windows"} {
		set initialdir "C:\\Program files\\"
		set filetypes {{"program files" {*.EXE *.COM}} {"batch files" {*.BAT *.CMD}} {"all files" {*.*}}}
	} else {
		set initialdir {/}
		set filetypes {{"all files" {* *.*}}}
	}
	if {$::toolscfg::edit_e2 != ""} {
		set initialdir [file dirname $::toolscfg::edit_e2]
		set initialfile [file tail $::toolscfg::edit_e2]
	} else {
		set initialfile ""
	}
	set f [tk_getOpenFile -initialdir $initialdir -initialfile $initialfile \
		-filetypes $filetypes -title "Tool configuration: select application"]
	if {$f != ""} {
		set ::toolscfg::edit_e2 [file nativename [file normalize $f]]
	}
	focus -force $w.top.b2
}
proc get_tool_workdir {parent} {
	set w .get_tool_workdir
	toplevel $w -class Dialog
	wm withdraw $w
	wm title $w "Tool working folder"
	wm transient $w $parent
	wm protocol $w WM_DELETE_WINDOW {set ::toolscfg::get_tool_workdir_done 0}
	bind $w <Escape> "set ::toolscfg::get_tool_workdir_done 0"

	button $w.b1 -text "VIDEO_TS folder (where the IFOs files are located)" \
			-command {set ::toolscfg::edit_e4 %p; set ::toolscfg::get_tool_workdir_done 1}
	button $w.b2 -text "DVD folder (if VIDEO_TS exists: parent folder of VIDEO_TS\nelse: where the IFOs files are located)" \
			-command {set ::toolscfg::edit_e4 %d; set ::toolscfg::get_tool_workdir_done 1}
	label  $w.l1 -padx 0 -pady 0 -justify left \
			-text "The two options above are used only when a DVD is loaded in PgcEdit.\nOtherwise, the application folder is used."
	button $w.b3 -text "Application folder" \
			-command {set ::toolscfg::edit_e4 %a; set ::toolscfg::get_tool_workdir_done 1}
	button $w.b4 -text "Select folder..." \
			-command "::toolscfg::selectworkdir $parent; set ::toolscfg::get_tool_workdir_done 1"
	button $w.b5 -text "No matter (keep current PgcEdit's working folder)" \
			-command {set ::toolscfg::edit_e4 ""; set ::toolscfg::get_tool_workdir_done 1}
	button $w.b6 -text "Cancel" \
			-command {set ::toolscfg::get_tool_workdir_done 0}
	pack  $w.b1 -fill x -padx 8 -pady 4
	pack  $w.b2 -fill x -padx 8 -pady 4
	pack  $w.l1 -fill x -padx 8 -anchor w
	pack  $w.b3 -fill x -padx 8 -pady 4
	pack  $w.b4 -fill x -padx 8 -pady 4
	pack  $w.b5 -fill x -padx 8 -pady 4
	pack  $w.b6 -fill x -padx 8 -pady 8

	update
	if {$::tcl_platform(platform) == "windows"} {
		catch {wm attributes $w -toolwindow 1}
	}
	wm minsize $w [winfo reqwidth $w] [winfo reqheight $w]
	wm maxsize $w [winfo reqwidth $w] [winfo reqheight $w]
	set wx [expr [winfo rootx $parent]+10]
	set wy [expr [winfo rooty $parent]+10]
	wm geometry $w +$wx+$wy
	wm deiconify $w
	grab set $w

	tkwait variable ::toolscfg::get_tool_workdir_done
	grab release $w
	grab set $parent
	destroy $w
}
proc selectworkdir {parent} {
	set initialdir [file dirname $::toolscfg::edit_e2]
	set d [tk_chooseDirectory -parent $parent -initialdir $initialdir \
			-title "Tool working folder"]
	if {$d != ""} {
		set ::toolscfg::edit_e4 [file nativename [file normalize $d]]
	}
}
proc args_info {} {
	if {$::tcl_platform(platform) == "windows"} {
		set dirsep "\\"
	} else {
		set dirsep "/"
	}
	set txt "The arguments are NOT used if there is no DVD loaded in PgcEdit!\n"

	set txt "$txt\nIn the arguments, you can use the following substitutions:\n"

	set txt "$txt\n  %p : VIDEO_TS folder path (where the IFO files are located)."
	set txt "$txt\n  %d : DVD folder path (parent of %p).  Same as %p if VIDEO_TS doesn't exist."
	set txt "$txt\n  %i : Current IFO filename: VIDEO_TS.IFO or VTS_##_0.IFO."
	set txt "$txt\n  %v : First video VOB for current VTS: VTS_##_1.VOB (or VIDEO_TS.VOB for VMGM)."
	set txt "$txt\n  %m : Menu VOB for current VTS: VIDEO_TS.VOB or VTS_##_0.VOB."
	set txt "$txt\n  %n : Number of TitleSets in DVD (ie. last Titleset number).\n"

	set txt "$txt\n  %D : Get folder path name interactively."
	set txt "$txt\n  %F : Get filename interactively (all file types)."
	set txt "$txt\n  %I : Get IFO filename interactively."
	set txt "$txt\n  %V : Get VOB filename interactively."
	set txt "$txt\n  %M : Get Menu VOB filename interactively."
	set txt "$txt\n  %T : Get text option interactively.\n"

	set txt "$txt\n  %% : % character.\n"

	set txt "$txt\nNotes:"
	set txt "$txt\n- %i, %v and %m are substitued by the filenames without any path."
	set txt "$txt\n- The interactive options are substitued with the full path."
	set txt "$txt\n- %v and %m are substitued even if the file doesn't exist."
	set txt "$txt\n- The filenames must be enclosed in double quotes.  Example: \"%p${dirsep}%i\""

	tk_messageBox -icon info -title "Tools Arguments infos" -message $txt
}
proc update_checkboxes {w} {
	if {$::toolscfg::background == "&"} {
		$w.ck3 configure -state disabled
		$w.ck4 configure -state disabled
		$w.ck5 configure -state disabled
		set ::toolscfg::reopen false
		set ::toolscfg::output false
		set ::toolscfg::iconify false
	} else {
		$w.ck3 configure -state normal
		$w.ck4 configure -state normal
		$w.ck5 configure -state normal
	}
}

#############################################################################
## save tools config
proc save_user_tools {w} {
	set str "# PgcEdit tools config file.  Saved automatically.  Do not edit!\n"
	set str "$str\nset ::config(tools) \{"
	foreach t $::config(tools) {
		set str "$str\n\t\{$t\}"
	}
	set str "$str\n\}"

	saveFile [file join $::config(config_directory) ".pgcedit.tools"] auto $str
}

#############################################################################
## configure main window Tools menu
proc configure_tools_menu {} {
	set toolidx 0
	set accelidx 1
	.mainmenu.tools delete 0 end
	foreach t $::config(tools) {
		if {[lindex $t 0]=="---"} {
			.mainmenu.tools add separator
		} else {
			if {$accelidx <= 12} {
				set acc "F[expr $accelidx]"
			} elseif {$accelidx <= 24} {
				set acc "Shift-F[expr $accelidx-12]"
			} elseif {$accelidx <= 36} {
				set acc "Control-F[expr $accelidx-24]"
			} else {
				set acc "Shift-Control-F[expr $accelidx-36]"
			}
			if {$accelidx != 25} {set break 0} else {set break 1}
			set tool [lindex $::config(tools) $toolidx]
			.mainmenu.tools add command -label [lindex $t 0] \
					-command "::toolscfg::exec_tool \{$tool\}" \
					-accelerator $acc -columnbreak $break
			bind . <$acc> "::toolscfg::exec_tool \{[string map {% %%} $tool]\}"
			incr accelidx
		}
		incr toolidx
	}
	.mainmenu.tools add separator
	.mainmenu.tools add command -label "Configure Tools" \
			-command "::toolscfg::toolscfg .toolscfg"
}

#############################################################################
## execute the selected tool
proc {exec_tool} {tool} {
	set toollab [lindex $tool 0]
	set appexec [lindex $tool 1]
	set appargs [lindex $tool 2]
	set bg      [lindex $tool 3]
	set tooldir [lindex $tool 4]
	set confirm [lindex $tool 5]
	set save    [lindex $tool 6]
	set reopen  [lindex $tool 7]
	set output  [lindex $tool 8]
	set iconify [lindex $tool 9]

	set oldpwd [pwd]
	set app  \"[string map {\\ \\\\} $appexec]\"
	if [info exists ::dvddir] {

		set dvddir   [file nativename $::dvddir]
		if {[file tail $::dvddir] == "VIDEO_TS"} {
			set dvdpath [file nativename [file dirname $::dvddir]]
		} else {
			set dvdpath $dvddir
		}
		if [info exists ::editor_cmds_from] {
			set vts [lindex [split [lindex $::editor_cmds_from 1] ","] 0]
			if {$vts == 0} {
				set ifofilename  "VIDEO_TS.IFO"
				set menufilename "VIDEO_TS.VOB"
				set vobfilename  "VIDEO_TS.VOB"
			} else {
				set index [format %02i $vts]
				set ifofilename  "VTS_${index}_0.IFO"
				set menufilename "VTS_${index}_0.VOB"
				set vobfilename  "VTS_${index}_1.VOB"
			}
		} else {
			set ifofilename  ""
			set menufilename ""
			set vobfilename  ""
		}
		set numvts $::pgcs(numvts)

		if {[string first "%D" $appargs] >= 0} {
			set d [tk_chooseDirectory -initialdir $::dvddir \
				-title "Select a folder for $toollab" -mustexist false]
			if {$d == ""} {return}
			set d [file nativename $d]
		} else {
			set d ""
		}
		if {[string first "%F" $appargs] >= 0} {
			set f [tk_getOpenFile -initialdir $::dvddir \
				-filetypes {{"all files" {*.*}} {"IFO and VOB files" {*.IFO *.VOB}} \
				{"IFO files" {*.IFO}} {"VOB files" {*.VOB}} {"Menu VOB files" {_TS.VOB _0.VOB}}} \
				-title "Select a file for $toollab"]
			if {$f == ""} {return}
			set f [file nativename $f]
		} else {
			set f ""
		}
		if {[string first "%I" $appargs] >= 0} {
			set i [tk_getOpenFile -initialdir $::dvddir \
				-filetypes {{"IFO files" {*.IFO}} {"all files" {*.*}}} \
				-title "Select an IFO file for $toollab"]
			if {$i == ""} {return}
			set i [file nativename $i]
		} else {
			set i ""
		}
		if {[string first "%V" $appargs] >= 0} {
			set v [tk_getOpenFile -initialdir $::dvddir \
				-filetypes {{"VOB files" {*.VOB}} {"Menu VOB files" {_TS.VOB _0.VOB}} \
				{"all files" {*.*}}} \
				-title "Select a VOB file for $toollab"]
			if {$v == ""} {return}
			set v [file nativename $v]
		} else {
			set v ""
		}
		if {[string first "%M" $appargs] >= 0} {
			set m [tk_getOpenFile -initialdir $::dvddir \
				-filetypes {{"Menu VOB files" {_TS.VOB _0.VOB}} {"VOB files" {*.VOB}} \
				{"all files" {*.*}}} \
				-title "Select a Menu VOB file for $toollab"]
			if {$m == ""} {return}
			set m [file nativename $m]
		} else {
			set m ""
		}
		if {[string first "%T" $appargs] >= 0} {
			set t [::stringreq::stringreq -window .gettextoption -text "Command line option for $toollab" \
				-title "PgcEdit tool: $toollab" -geometry c -cancelreturn "***CANCELLED***"]
			if {$t == "***CANCELLED***"} {return}
		} else {
			set t ""
		}
		set args [string map [list %% % %p $dvddir %d $dvdpath %i $ifofilename %m $menufilename \
				%n $numvts %v $vobfilename %D $d %F $f %I $i %V $v %M $m %T $t] $appargs]

		set workdir $dvddir
		if {$tooldir == "%d"} {
			set workdir $dvdpath
		}
	} else {
		set args ""
		set workdir [file dirname $appexec]
	}
	set workdiropt $tooldir
	if {$workdiropt != "%p" && $workdiropt != "%d"} {
		if {$workdiropt == "%a"} {
			set workdir [file dirname $appexec]
		} else {
			set workdir [string map {%% %} $workdiropt]
		}
		if {$workdir == ""} {set workdir .}
	}
	set workdir [file nativename [file normalize $workdir]]

	if $confirm {
		set r [tk_messageBox -icon question -title "PgcEdit: User tool" -type yesno \
			-message "Execute tool \"$toollab\" ?\n\nCommand line:\n\"$appexec\" $args $bg" \
			-parent .]
		if {$r == "no"} {
			return
		}
	}

	if {[catch {cd $workdir} errtxt] != 0} {
		set r [tk_messageBox -icon error -title "PgcEdit: User tool error" -type yesno \
			-message "Error changing working directory\nfor user tool \"$toollab\"\nto \"$workdir\":\n$errtxt\n\nExecute tool anyway?" \
			-parent .]
		if {$r == "no"} {
			cd $oldpwd
			return
		}
	}

	if {$save==true && [info exists ::dvddir] && $::dvd_modified} {
		if {[::main::Write_DVD false] > 0} {
			cd $oldpwd
			return
		}
	}

	if $iconify {wm iconify .}
	set args [string map [list \\ \\\\] $args]
	set err [catch {set txt [eval exec -- $app $args $bg]} errtxt]
	if $iconify {wm deiconify .}
	update idletasks
	if {$err > 0} {
		show_warning -icon error -title "PgcEdit tool: $toollab" \
			-message "Error executing user tools \"$toollab\".\n\n$errtxt" -parent .
	} elseif {$output && $txt != ""} {
		show_warning -icon info -title "PgcEdit tool: $toollab" \
			-message "Tools \"$toollab\" output:\n\n$txt" -parent .
	}

	if {$reopen && [info exists ::dvddir]} {
		update idletasks
		::main::ReRead_DVD
	}
	cd $oldpwd
}

} ;# end of namespace ::toolscfg
